################################################################################
# ECDC OPERATIONAL SUPPORT Tools and methods for promoting                     #
# vaccination acceptance and uptake: a social and behavioural science approach #
################################################################################

# To use this analysis code, ensure that the variables are named according to 
# the following list:
# Demographic variables:
# age, gender, education, region, employment_status

# Vaccination behavioural outcomes:
# vaccinated, refused, intend

# 5Cs indicators:
# Confidence: conf1, conf2, conf3
# Complacency: comp1, comp2, comp3
# Constraints: cons1, cons2, cons3
# Calculation: calc1, calc2, calc3
# Collective responsibility: coll1, coll2, coll3

# 1. Load libraries ----
# Install packages
# install.packages(c("dplyr", "forcats", "haven", "nnet", 
#                    "janitor", "broom", "MASS", "tidyr"))

# Load libraries 
library(dplyr)    # Data wrangling
library(forcats)  # Factor handling
library(haven)    # Read stata
library(nnet)     # Multinomial models
library(janitor)  # Clean data
library(broom)    # Tidy model output
library(MASS)     # Stats & datasets
library(tidyr)    # Tidy data reshaping 

# 2. Data ----
## 2.1 Load data ----
# Load example dataset (to be updated)
df <- read_dta("example_data.dta")

## 2.2 Data preparation ----
# Set missing values for specific codes
# for "I do not know/Prefer not to say"
df <- df %>%
  mutate(across(c(vaccinated, refused, intend, 
                  conf1:conf3, comp1:comp3, 
                  cons1:cons3, calc1:calc3, 
                  coll1:coll3),
                ~ na_if(., 99)),
         gender = na_if(gender, 4))

# Reverse-code specified for 
# comp1, cons2, cons3, calc2, calc3, coll2 
df <- df %>%
  mutate(comp1 = 6 - comp1,
         cons2 = 6 - cons2,
         cons3 = 6 - cons3,
         calc2 = 6 - calc2,
         calc3 = 6 - calc3,
         coll2 = 6 - coll2)

# Categorise age
df <- df %>%
  mutate(age_group = case_when(age >= 18 & age <= 25 ~ 1,
                               age >= 26 & age <= 40 ~ 2,
                               age >= 41 & age <= 64 ~ 3,
                               age >= 65 ~ 4))

# Labeling variables and values
df <- df %>%
  mutate(age_group = factor(age_group, 
                            labels = c("18-25", "26-40", "41-64", "65+")),
         gender = factor(gender, 
                         labels = c("Male", "Female", "Non-binary")),
         education = factor(education, 
                            labels = c("Low", "Medium", "High")),
         employment_status = factor(employment_status, 
                                    labels = c("Employed", "Self-Employed", 
                                               "Out of work", "Student", 
                                               "Retired", "Unable to work", "Other")),
         vaccinated = factor(vaccinated, 
                             labels = c("None", "Some", "All")),
         refused = factor(refused, labels = c("None", "Some", "All")),
         intend = factor(intend, levels = 1:5, 
                         labels = c("Definitely not", "Probably not", "Maybe", 
                                    "Probably yes", "Definitely yes")))

# Generate mean scores on the 5Cs
df <- df %>%
  mutate(confidence = rowMeans(cbind(conf1, conf2, conf3), na.rm = TRUE),
         complacency = rowMeans(cbind(comp1, comp2, comp3), na.rm = TRUE),
         constraints = rowMeans(cbind(cons1, cons2, cons3), na.rm = TRUE),
         calculation = rowMeans(cbind(calc1, calc2, calc3), na.rm = TRUE),
         collective_responsibility = rowMeans(cbind(coll1, coll2, coll3), 
                                              na.rm = TRUE))

# Create 3-category intend variable
df <- df %>%
  mutate(intend_3cat = case_when(
    intend %in% c("Definitely not", "Probably not") ~ "Def./prob. not",
    intend == "Maybe" ~ "Neutral",
    intend %in% c("Probably yes", "Definitely yes") ~ "Prob./Def. yes") %>% 
      factor(levels = c("Def./prob. not", "Neutral", "Prob./Def. yes")))

# 3. Descriptive tables (Counts + Percentages with row and column totals) ----
## 3.1 % vaccinated as recommended by age, gender, education, and region ----
df %>%
  tabyl(vaccinated) %>% 
  adorn_totals(where = "row") %>% 
  adorn_pct_formatting(digits = 1)             # convert proportions to percents

# age
df %>%
  tabyl(age_group, vaccinated) %>%
  adorn_totals(where = "col") %>%              # add total row
  adorn_percentages(denominator = "row") %>%   # convert counts to proportions
  adorn_pct_formatting(digits = 1)   %>%       # convert proportions to percents
  adorn_ns()                                   # add counts alongside percentages

df %>%
  tabyl(age_group, vaccinated) %>%
  adorn_totals(where = "row") %>%              # add total col
  adorn_percentages(denominator = "col") %>%   # convert counts to proportions
  adorn_pct_formatting(digits = 1)   %>%       # convert proportions to percents
  adorn_ns()                                   # add counts alongside percentages

# gender
df %>%
  tabyl(gender, vaccinated) %>%
  adorn_totals(where = "col") %>%              # add total row
  adorn_percentages(denominator = "row") %>%   # convert counts to proportions
  adorn_pct_formatting(digits = 1)   %>%       # convert proportions to percents
  adorn_ns()                                   # add counts alongside percentages

df %>%
  tabyl(gender, vaccinated) %>%
  adorn_totals(where = "row") %>%              # add total col
  adorn_percentages(denominator = "col") %>%   # convert counts to proportions
  adorn_pct_formatting(digits = 1)   %>%       # convert proportions to percents
  adorn_ns()                                   # add counts alongside percentages

# education
df %>%
  tabyl(education, vaccinated) %>%
  adorn_totals(where = "col") %>%              # add total row
  adorn_percentages(denominator = "row") %>%   # convert counts to proportions
  adorn_pct_formatting(digits = 1)   %>%       # convert proportions to percents
  adorn_ns()                                   # add counts alongside percentages

df %>%
  tabyl(education, vaccinated) %>%
  adorn_totals(where = "row") %>%              # add total col
  adorn_percentages(denominator = "col") %>%   # convert counts to proportions
  adorn_pct_formatting(digits = 1)   %>%       # convert proportions to percents
  adorn_ns()                                   # add counts alongside percentages

# region
df %>%
  tabyl(region, vaccinated) %>%
  adorn_totals(where = "col") %>%              # add total row
  adorn_percentages(denominator = "row") %>%   # convert counts to proportions
  adorn_pct_formatting(digits = 1)   %>%       # convert proportions to percents
  adorn_ns()                                   # add counts alongside percentages

df %>%
  tabyl(region, vaccinated) %>%
  adorn_totals(where = "row") %>%              # add total col
  adorn_percentages(denominator = "col") %>%   # convert counts to proportions
  adorn_pct_formatting(digits = 1)   %>%       # convert proportions to percents
  adorn_ns() 

## 3.2 % refused recommended vaccinations by age, gender, education, and region ----
df %>%
  tabyl(refused) %>% 
  adorn_totals(where = "row") %>% 
  adorn_pct_formatting(digits = 1)             # convert proportions to percents

# age
df %>%
  tabyl(age_group, refused) %>%
  adorn_totals(where = "col") %>%              # add total row
  adorn_percentages(denominator = "row") %>%   # convert counts to proportions
  adorn_pct_formatting(digits = 1)   %>%       # convert proportions to percents
  adorn_ns()                                   # add counts alongside percentages

df %>%
  tabyl(age_group, refused) %>%
  adorn_totals(where = "row") %>%              # add total col
  adorn_percentages(denominator = "col") %>%   # convert counts to proportions
  adorn_pct_formatting(digits = 1)   %>%       # convert proportions to percents
  adorn_ns()                                   # add counts alongside percentages

# gender
df %>%
  tabyl(gender, refused) %>%
  adorn_totals(where = "col") %>%              # add total row
  adorn_percentages(denominator = "row") %>%   # convert counts to proportions
  adorn_pct_formatting(digits = 1)   %>%       # convert proportions to percents
  adorn_ns()                                   # add counts alongside percentages

df %>%
  tabyl(gender, refused) %>%
  adorn_totals(where = "row") %>%              # add total col
  adorn_percentages(denominator = "col") %>%   # convert counts to proportions
  adorn_pct_formatting(digits = 1)   %>%       # convert proportions to percents
  adorn_ns()                                   # add counts alongside percentages

# education
df %>%
  tabyl(education, refused) %>%
  adorn_totals(where = "col") %>%              # add total row
  adorn_percentages(denominator = "row") %>%   # convert counts to proportions
  adorn_pct_formatting(digits = 1)   %>%       # convert proportions to percents
  adorn_ns()                                   # add counts alongside percentages

df %>%
  tabyl(education, refused) %>%
  adorn_totals(where = "row") %>%              # add total col
  adorn_percentages(denominator = "col") %>%   # convert counts to proportions
  adorn_pct_formatting(digits = 1)   %>%       # convert proportions to percents
  adorn_ns()                                   # add counts alongside percentages

# region
df %>%
  tabyl(region, refused) %>%
  adorn_totals(where = "col") %>%              # add total row
  adorn_percentages(denominator = "row") %>%   # convert counts to proportions
  adorn_pct_formatting(digits = 1)   %>%       # convert proportions to percents
  adorn_ns()                                   # add counts alongside percentages

df %>%
  tabyl(region, refused) %>%
  adorn_totals(where = "row") %>%              # add total col
  adorn_percentages(denominator = "col") %>%   # convert counts to proportions
  adorn_pct_formatting(digits = 1)   %>%       # convert proportions to percents
  adorn_ns() 

## 3.3 % intend to get vaccinated by age, gender, education, and region ----
df %>%
  tabyl(intend) %>% 
  adorn_totals(where = "row") %>% 
  adorn_pct_formatting(digits = 1)             # convert proportions to percents

# age
df %>%
  tabyl(age_group, intend) %>%
  adorn_totals(where = "col") %>%              # add total row
  adorn_percentages(denominator = "row") %>%   # convert counts to proportions
  adorn_pct_formatting(digits = 1)   %>%       # convert proportions to percents
  adorn_ns()                                   # add counts alongside percentages

df %>%
  tabyl(age_group, intend) %>%
  adorn_totals(where = "row") %>%              # add total col
  adorn_percentages(denominator = "col") %>%   # convert counts to proportions
  adorn_pct_formatting(digits = 1)   %>%       # convert proportions to percents
  adorn_ns()                                   # add counts alongside percentages

# gender
df %>%
  tabyl(gender, intend) %>%
  adorn_totals(where = "col") %>%              # add total row
  adorn_percentages(denominator = "row") %>%   # convert counts to proportions
  adorn_pct_formatting(digits = 1)   %>%       # convert proportions to percents
  adorn_ns()                                   # add counts alongside percentages

df %>%
  tabyl(gender, intend) %>%
  adorn_totals(where = "row") %>%              # add total col
  adorn_percentages(denominator = "col") %>%   # convert counts to proportions
  adorn_pct_formatting(digits = 1)   %>%       # convert proportions to percents
  adorn_ns()                                   # add counts alongside percentages

# education
df %>%
  tabyl(education, intend) %>%
  adorn_totals(where = "col") %>%              # add total row
  adorn_percentages(denominator = "row") %>%   # convert counts to proportions
  adorn_pct_formatting(digits = 1)   %>%       # convert proportions to percents
  adorn_ns()                                   # add counts alongside percentages

df %>%
  tabyl(education, intend) %>%
  adorn_totals(where = "row") %>%              # add total col
  adorn_percentages(denominator = "col") %>%   # convert counts to proportions
  adorn_pct_formatting(digits = 1)   %>%       # convert proportions to percents
  adorn_ns()                                   # add counts alongside percentages

# region
df %>%
  tabyl(region, intend) %>%
  adorn_totals(where = "col") %>%              # add total row
  adorn_percentages(denominator = "row") %>%   # convert counts to proportions
  adorn_pct_formatting(digits = 1)   %>%       # convert proportions to percents
  adorn_ns()                                   # add counts alongside percentages

df %>%
  tabyl(region, intend) %>%
  adorn_totals(where = "row") %>%              # add total col
  adorn_percentages(denominator = "col") %>%   # convert counts to proportions
  adorn_pct_formatting(digits = 1)   %>%       # convert proportions to percents
  adorn_ns() 


# 4. 5C indicator agreement ----
# Core Indicators: Confidence, Complacency, Constraints,
# Calculation, Collective Responsibility
## 4.1  % agreeing with key 5C indicators (answered 4 or 5) ----
for (var in c("conf1", "comp1", "cons1", "calc1", "coll1")) {
  agree_var <- paste0(var, "_agree")
  df[[agree_var]] <- df[[var]] %in% 4:5
}

df %>% 
  summarise(across(ends_with("_agree"), ~ mean(., na.rm = TRUE)))

df %>%
  dplyr::select(contains("confidence"), contains("complacency"), contains("constraints"),
                contains("calculation"), contains("collective_responsibility")) %>%
  pivot_longer(everything(), names_to = "Variable", values_to = "Value") %>%
  group_by(Variable) %>%
  summarise(Obs = sum(!is.na(Value)),
            Mean = mean(Value, na.rm = TRUE),
            SD = sd(Value, na.rm = TRUE),
            Min = min(Value, na.rm = TRUE),
            Max = max(Value, na.rm = TRUE),
            .groups = "drop")



## 4.2 Mean scores of each C stratified by demographic variables ----
# Define common variables
vars <- c("confidence", "complacency", "constraints", "calculation", "collective_responsibility")

# By age_group
summary_by_age_group <- df %>%
  dplyr::select(age_group, all_of(vars)) %>%
  pivot_longer(cols = all_of(vars), names_to = "Variable", values_to = "Value") %>%
  group_by(age_group, Variable) %>%
  summarise(Obs = sum(!is.na(Value)),
            Mean = mean(Value, na.rm = TRUE),
            SD = sd(Value, na.rm = TRUE),
            Min = min(Value, na.rm = TRUE),
            Max = max(Value, na.rm = TRUE),
            .groups = "drop")
summary_by_age_group

# By gender
summary_by_gender <- df %>%
  dplyr::select(gender, all_of(vars)) %>%
  pivot_longer(cols = all_of(vars), names_to = "Variable", values_to = "Value") %>%
  group_by(gender, Variable) %>%
  summarise(Obs = sum(!is.na(Value)),
            Mean = mean(Value, na.rm = TRUE),
            SD = sd(Value, na.rm = TRUE),
            Min = min(Value, na.rm = TRUE),
            Max = max(Value, na.rm = TRUE),
            .groups = "drop")
summary_by_gender

# By education
summary_by_education <- df %>%
  dplyr::select(education, all_of(vars)) %>%
  pivot_longer(cols = all_of(vars), names_to = "Variable", values_to = "Value") %>%
  group_by(education, Variable) %>%
  summarise(Obs = sum(!is.na(Value)),
            Mean = mean(Value, na.rm = TRUE), 
            SD = sd(Value, na.rm = TRUE), 
            Min = min(Value, na.rm = TRUE),
            Max = max(Value, na.rm = TRUE),
            .groups = "drop")
summary_by_education

# By region
summary_by_region <- df %>%
  dplyr::select(region, all_of(vars)) %>%
  pivot_longer(cols = all_of(vars), names_to = "Variable", values_to = "Value") %>%
  group_by(region, Variable) %>%
  summarise(Obs = sum(!is.na(Value)),
            Mean = mean(Value, na.rm = TRUE),
            SD = sd(Value, na.rm = TRUE),
            Min = min(Value, na.rm = TRUE),
            Max = max(Value, na.rm = TRUE),
            .groups = "drop")
summary_by_region

# By employment_status
summary_by_employment <- df %>%
  dplyr::select(employment_status, all_of(vars)) %>%
  pivot_longer(cols = all_of(vars), names_to = "Variable", values_to = "Value") %>%
  group_by(employment_status, Variable) %>%
  summarise(Obs = sum(!is.na(Value)),
            Mean = mean(Value, na.rm = TRUE),
            SD = sd(Value, na.rm = TRUE),
            Min = min(Value, na.rm = TRUE),
            Max = max(Value, na.rm = TRUE),
            .groups = "drop")
summary_by_employment


# 5. Inferential analyses ----
# Ensure factors
df <- df %>%
  mutate(age_group = factor(age_group),
         gender = factor(gender),
         education = factor(education),
         region = factor(region),
         employment_status = factor(employment_status))

## NOTE: Regression analysis for predictors of vaccination. 
# The choice of the model (i.e., ordinal logistic, multinomial or 
# binary logistic regression) depends on theoretical and practical considerations

## 5.1 Linear regression ----
# Socio-demographic predictors for each C (Linear Regression)
summary(lm(confidence ~ age_group + gender + education + region + employment_status, data = df))
summary(lm(complacency ~ age_group + gender + education + region + employment_status, data = df))
summary(lm(constraints ~ age_group + gender + education + region + employment_status, data = df))
summary(lm(calculation ~ age_group + gender + education + region + employment_status, data = df))
summary(lm(collective_responsibility ~ age_group + gender + education + region + employment_status, data = df))

## 5.2 Logistic regression ----
# Binary logistic regression for vaccination intention, refusal, and being vaccinated
df <- df %>%
  mutate(intend_binary = case_when(intend %in% c("Definitely not", "Probably not", "Maybe") ~ 0,
                                   intend %in% c("Probably yes", "Definitely yes") ~ 1),
         vaccinated_binary = ifelse(vaccinated == "All", 1, 0),
         refused_binary = ifelse(refused == "All", 1, 0))

# intend_binary
summary(glm(intend_binary ~ age_group + gender + education + region + employment_status +
              confidence + complacency + constraints + calculation + collective_responsibility,
            data = df, family = binomial()))

tidy(glm(intend_binary ~ age_group + gender + education + region + employment_status +
           confidence + complacency + constraints + calculation + collective_responsibility,
         data = df, family = binomial()),
     conf.int = TRUE, exponentiate = TRUE) %>%
  dplyr::select(term, estimate, conf.low, conf.high, p.value) %>%
  rename(OR = estimate, CI_low = conf.low, CI_high = conf.high, p = p.value)

# refused_binary
summary(glm(refused_binary ~ age_group + gender + education + region + employment_status +
              confidence + complacency + constraints + calculation + collective_responsibility,
            data = df, family = binomial()))

tidy(glm(refused_binary ~ age_group + gender + education + region + employment_status +
           confidence + complacency + constraints + calculation + collective_responsibility,
         data = df, family = binomial()),
     conf.int = TRUE, exponentiate = TRUE) %>%
  dplyr::select(term, estimate, conf.low, conf.high, p.value) %>%
  rename(OR = estimate, CI_low = conf.low, CI_high = conf.high, p = p.value)

# vaccinated_binary
summary(glm(vaccinated_binary ~ age_group + gender + education + region + employment_status +
              confidence + complacency + constraints + calculation + collective_responsibility,
            data = df, family = binomial()))

tidy(glm(vaccinated_binary ~ age_group + gender + education + region + employment_status +
           confidence + complacency + constraints + calculation + collective_responsibility,
         data = df, family = binomial()),
     conf.int = TRUE, exponentiate = TRUE) %>%
  dplyr::select(term, estimate, conf.low, conf.high, p.value) %>%
  rename(OR = estimate, CI_low = conf.low, CI_high = conf.high, p = p.value)


## 5.3 Ordinal logistic regression for vaccination intention, refusal, and being vaccinated ----
# Prepare the data
df <- df %>%
  mutate(intend = ordered(intend),  # Make sure levels are in correct order
         refused = ordered(refused),
         vaccinated = ordered(vaccinated),
         age_group = factor(age_group),
         gender = factor(gender),
         education = factor(education),
         region = factor(region),
         employment_status = factor(employment_status))

# Fit ordinal logistic regression models
intend_model <- polr(intend ~ age_group + gender + education + region + employment_status +
                       confidence + complacency + constraints + calculation + collective_responsibility,
                     data = df, Hess = TRUE)

summary(intend_model)

refused_model <- polr(refused ~ age_group + gender + education + region + employment_status +
                        confidence + complacency + constraints + calculation + collective_responsibility,
                      data = df, Hess = TRUE)

summary(refused_model)

vaccinated_model <- polr(vaccinated ~ age_group + gender + education + region + employment_status +
                           confidence + complacency + constraints + calculation + collective_responsibility,
                         data = df, Hess = TRUE)

summary(vaccinated_model)

# Helper: Extract ORs and 95% CI with manually computed p-values
get_or_ci <- function(model) {
  tidy(model, conf.int = TRUE, exponentiate = TRUE) %>%
    filter(!grepl("\\|", term)) %>%
    mutate(
      p = 2 * (1 - pnorm(abs(statistic)))) %>%
    dplyr::select(term, estimate, conf.low, conf.high, p) %>%
    dplyr::rename(
      OR = estimate,
      CI_low = conf.low,
      CI_high = conf.high)
}

# Extract results
intend_or <- get_or_ci(intend_model)
refused_or <- get_or_ci(refused_model)
vaccinated_or <- get_or_ci(vaccinated_model)

# View results
intend_or
refused_or
vaccinated_or


## 5.4  Multinomial logistic regression for vaccination intention, refusal, and being vaccinated ----
# Prepare the data with correct reference levels
df <- df %>%
  mutate(intend_3cat = relevel(factor(intend_3cat), ref = "Def./prob. not"),
         refused = relevel(factor(as.character(refused)), ref = "None"),
         vaccinated = relevel(factor(as.character(vaccinated)), ref = "None"),
         age_group = factor(age_group),
         gender = factor(gender),
         education = factor(education),
         region = factor(region),
         employment_status = factor(employment_status))


# Fit the multinomial logistic regression models
intend_model <- multinom(intend_3cat ~ age_group + gender + education + region + employment_status +
                           confidence + complacency + constraints + calculation + collective_responsibility,
                         data = df)
summary(intend_model)

refused_model <- multinom(refused ~ age_group + gender + education + region + employment_status +
                            confidence + complacency + constraints + calculation + collective_responsibility,
                          data = df)

vaccinated_model <- multinom(vaccinated ~ age_group + gender + education + region + employment_status +
                               confidence + complacency + constraints + calculation + collective_responsibility,
                             data = df)

# Helper function to extract RRRs, CIs, and p-values
get_rrr <- function(model) {
  tidy(model, exponentiate = TRUE, conf.int = TRUE) %>%
    mutate(p = 2 * (1 - pnorm(abs(statistic)))) %>%
    dplyr::select(y.level, term, estimate, conf.low, conf.high, p) %>%
    dplyr::rename(
      Category = y.level,
      RRR = estimate,
      CI_low = conf.low,
      CI_high = conf.high)
}

# Extract results for each model
intend_rrr <- get_rrr(intend_model)
refused_rrr <- get_rrr(refused_model)
vaccinated_rrr <- get_rrr(vaccinated_model)

# View or export results
print(intend_rrr, n=44)
print(refused_rrr, n=44)
print(vaccinated_rrr, n=44)
